@@ -350,7 +350,7 @@ impl FromStr for DescriptorPublicKey {
 }
 
 /// Descriptor key conversion error
-#[derive(Debug, PartialEq, Clone, Copy)]
+#[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Hash, Clone, Copy)]
 pub enum ConversionError {
     /// Attempted to convert a key with a wildcard to a bitcoin public key
     Wildcard,
@@ -626,6 +626,7 @@ impl<K: InnerXKey> DescriptorXKey<K> {
     /// ## Examples
     ///
     /// ```
+    /// # extern crate sapio_miniscript as miniscript;
     /// # use std::str::FromStr;
     /// # fn body() -> Result<(), Box<dyn std::error::Error>> {
     /// use miniscript::bitcoin::util::bip32;
