@@ -46,8 +46,3 @@ impl MsKeyBuilder for script::Builder {
         }
     }
 }
-
-// This belongs in rust-bitcoin, make this upstream later
-pub(crate) fn script_is_v1_tr(s: &Script) -> bool {
-    s.is_v1_p2tr()
-}
